using System;

namespace PdnHSL
{
    /// <summary>
    /// The code of RGB/HSL conversions. For best performances, this
    /// code may not be called as methods but copied where needed.
    /// </summary>
    public static class HslHelper
    {
        /// <summary>
        /// Converts from RGB to HSL
        /// </summary>
        /// <param name="r">Red : 0..255</param>
        /// <param name="g">Green : 0..255</param>
        /// <param name="b">Blue : 0..255</param>
        /// <param name="h">Hue : 0..359</param>
        /// <param name="s">Saturation : 0..255</param>
        /// <param name="l">Lightness : 0..255</param>
        /// <remarks>
        /// Maximum difference of results compared to using doubles :
        /// h : 1 (avg : 0.05)
        /// s : 1 (avg : 0.5)
        /// l : 1 (avg : 0.2)
        /// </remarks>
        public static void GetHSL(int r, int g, int b, out int h, out int s, out int l)
        {
            Lookup lut = Lookup.Current;

            int min = r < g ? (r < b ? r : b) : (g < b ? g : b);
            int max = r > g ? (r > b ? r : b) : (g > b ? g : b);
            int delta = max - min;

            l = (max + min) >> 1;

            if (max == 0 || delta == 0)
            {
                h = s = 0;
            }
            else
            {
                if (l == 0)
                {
                    s = 255;
                }
                else if (l < 128)
                {
                    s = (lut.PercentDivide[delta | (l << 8)] >> 1);
                }
                else
                {
                    s = (lut.PercentDivide[delta | ((255 - l) << 8)] >> 1) + 1;

                    if (l > 188) ++s;           // This hack regulates the value so that there
                    if (l > 213) ++s;           // are no big difference between this algorithm
                    if (l > 223) ++s;           // and the good algorithm. Without this, the 
                    if (l > 228) ++s;           // maximum difference would be 40 (instead of 1).
                    if (l > 236)                // Tested with all possible RGB values.
                        s += (l - 235) << 1;
                }

                if (s > 255) s = 255;

                if (r == max)
                {
                    h = g - b;
                    if (h < 0)
                    {
                        h = -lut.PercentDivide[-h | (delta << 8)];
                    }
                    else
                    {
                        h = lut.PercentDivide[h | (delta << 8)];
                    }

                }
                else if (g == max)
                {
                    h = b - r;
                    if (h < 0)
                    {
                        h = 510 - lut.PercentDivide[-h | (delta << 8)];
                    }
                    else
                    {
                        h = 510 + lut.PercentDivide[h | (delta << 8)];
                    }
                }
                else
                {
                    h = r - g;
                    if (h < 0)
                    {
                        h = 1020 - lut.PercentDivide[-h | (delta << 8)];
                    }
                    else
                    {
                        h = 1020 + lut.PercentDivide[h | (delta << 8)];
                    }
                }

                if (h >= 1530) h -= 1530;
                if (h < 0) h += 1530;

                h = lut.HSL_Scale1530To360[h];
            }
        }

        /// <summary>
        /// Converts from HSL to RGB
        /// </summary>
        /// <param name="h">Hue : 0..359</param>
        /// <param name="s">Saturation : 0..255</param>
        /// <param name="l">Lightness : 0..255</param>
        /// <param name="r">Red : 0..255</param>
        /// <param name="g">Green :0..255</param>
        /// <param name="b">Blue : 0..255</param>
        /// <remarks>
        /// Maximum difference of results compared to using doubles :
        /// r, g, b : 2 (avg : 0.4)
        /// </remarks>
        public static void GetRGB(int h, int s, int l, out int r, out int g, out int b)
        {
            Lookup lut = Lookup.Current;

            int q, p, hk;
            int tr, tg, tb;

            if (l < 128)
            {
                q = l + lut.PercentMultiply[l | (s << 8)];
            }
            else
            {
                q = (l + s) - lut.PercentMultiply[(int)l | ((int)s << 8)];
            }

            p = (l << 1) - q;

            hk = lut.HSL_Scale360To255[h];

            tr = hk + 85;
            tg = hk;
            tb = hk - 85;

            if (tr < 0) tr += 255;
            if (tr > 255) tr -= 255;

            if (tb < 0) tb += 255;
            if (tb > 255) tb -= 255;

            if (tr < 43)
            {
                r = p + lut.PercentMultiply[(q - p) | (lut.HSL_MultiplyBy6[tr] << 8)];
            }
            else if (tr < 128)
            {
                r = q;
            }
            else if (tr < 170)
            {
                r = p + lut.PercentMultiply[(q - p) | (lut.HSL_MultiplyBy6[170 - tr] << 8)];
            }
            else
            {
                r = p;
            }

            if (tg < 43)
            {
                g = p + lut.PercentMultiply[(q - p) | (lut.HSL_MultiplyBy6[tg] << 8)];
            }
            else if (tg < 128)
            {
                g = q;
            }
            else if (tg < 170)
            {
                g = p + lut.PercentMultiply[(q - p) | (lut.HSL_MultiplyBy6[170 - tg] << 8)];
            }
            else
            {
                g = p;
            }

            if (tb < 43)
            {
                b = p + lut.PercentMultiply[(q - p) | (lut.HSL_MultiplyBy6[tb] << 8)];
            }
            else if (tb < 128)
            {
                b = q;
            }
            else if (tb < 170)
            {
                b = p + lut.PercentMultiply[(q - p) | (lut.HSL_MultiplyBy6[170 - tb] << 8)];
            }
            else
            {
                b = p;
            }
        }
    }
}
