using System;

namespace PdnHSL
{
    /// <summary>
    /// Contains the required lookup-tables for fast RGB/HSL conversion
    /// </summary>
    public class Lookup
    {
        #region Tables

        /// <summary>
        /// Multiply bytes as if it was 0..1 values
        /// = (byte)Math.Round((((double)i / 255d) * ((double)j / 255d)) * 255d)
        /// </summary>
        public readonly byte[] PercentMultiply;

        /// <summary>
        /// Divide bytes as if it was 0..1 values
        /// = (short)Math.Round((((double)i / 255d) / ((double)j / 255d)) * 255d)
        /// </summary>
        public readonly short[] PercentDivide;

        /// <summary>
        /// Scale from 0..1530 to 0..360
        /// = (short)Math.Round((double)i / 4.25d);
        /// </summary>
        public readonly short[] HSL_Scale1530To360;

        /// <summary>
        /// Multiply by 6.07142857142857, useful because we use integers and
        /// 42.5 (1/6 * 255) is not an integer, so we would do 6 * 42 = 252
        /// which is not good because the max value must be 255
        /// = (byte)Math.Min(255, Math.Round((double)i * 6.07142857142857d));
        /// </summary>
        public readonly byte[] HSL_MultiplyBy6;

        /// <summary>
        /// Scale from 0..360 to 0..255
        /// = (byte)Math.Round(((double)i / 360d) * 255d);
        /// </summary>
        public readonly byte[] HSL_Scale360To255;

        #endregion

        #region Initialization

        /// <summary>
        /// Initialize the Lookup tables
        /// </summary>
        private Lookup()
        {
            PercentMultiply = new byte[65536];
            PercentDivide = new short[65536];
            HSL_Scale1530To360 = new short[1531];
            HSL_MultiplyBy6 = new byte[170];
            HSL_Scale360To255 = new byte[361];

            for (int i = 0; i < 170; ++i)
            {
                HSL_MultiplyBy6[i] = (byte)Math.Min(255, Math.Round((double)i * 6.07142857142857d));
            }

            for (int i = 0; i < 361; ++i)
            {
                HSL_Scale360To255[i] = (byte)Math.Round(((double)i / 360d) * 255d);
            }

            for (int i = 0; i < 1531; ++i)
            {
                HSL_Scale1530To360[i] = (short)(Math.Round((double)i / 4.25d));
                if (HSL_Scale1530To360[i] == 360) HSL_Scale1530To360[i] = 0;
            }

            for (int i = 0; i < 256; ++i)
            {
                for (int j = 0; j < 256; ++j)
                {
                    int jby256 = j << 8;

                    PercentMultiply[i + jby256] = (byte)Math.Round((((double)i / 255d) * ((double)j / 255d)) * 255d);
                    PercentDivide[i + jby256] = (short)Math.Round((((double)i / 255d) / ((double)j / 255d)) * 255d);

                }
            }

        }

        #endregion

        #region Singleton

        public static readonly Lookup Current;

        static Lookup()
        {
            Current = new Lookup();
        }

        #endregion
    }
}
